@@ -537,6 +537,30 @@ is_workgroup_server_link (const char *workgroup,
 
 }
 
+char *
+get_workgroup_data (char *name)
+{
+  return g_strdup_printf ("[Desktop Entry]\n"
+			  "Encoding=Legacy-Mixed\n"
+			  "Name=%s\n"
+			  "Type=Link\n"
+			  "URL=smb://%s/\n"
+			  "Icon=gnome-computer.png\n",
+			  name, name);
+}
+
+char *
+get_computer_data (char *name)
+{
+  return g_strdup_printf ("[Desktop Entry]\n"
+			  "Encoding=Legacy-Mixed\n"
+			  "Name=%s\n"
+			  "Type=Link\n"
+			  "URL=smb://%s/\n"
+			  "Icon=gnome-computer.png\n",
+			  name, name);
+}
+
 static GnomeVFSResult
 do_open (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle **method_handle,
@@ -584,19 +608,13 @@ do_open (GnomeVFSMethod *method,
 		handle = g_new (SmbHandle, 1);
 		handle->is_data = TRUE;
 		handle->offset = 0;
-		handle->file_data =
-			g_strdup_printf ("<?xml version=\"1.0\"?>\n"
-					 "<nautilus_object nautilus_link=\"Nautilus Link\" custom_icon=\"gnome-amusements.png\" link=\"smb://%s/\"/>",
-					 (*path=='/')?path+1:path);
+		handle->file_data = get_workgroup_data ((*path=='/')?path+1:path);
 		handle->offset = 0;
 	} else if (is_workgroup_server_link (server, path)) {
 		handle = g_new (SmbHandle, 1);
 		handle->is_data = TRUE;
 		handle->offset = 0;
-		handle->file_data =
-			g_strdup_printf ("<?xml version=\"1.0\"?>\n"
-					 "<nautilus_object nautilus_link=\"Nautilus Link\" custom_icon=\"gnome-amusements.png\" link=\"smb://%s/\"/>",
-					 (*path=='/')?path+1:path);
+		handle->file_data = get_computer_data ((*path=='/')?path+1:path);
 		handle->offset = 0;
 	} else {
 		res = smb_server_connection_new_from_uri (uri,
@@ -987,6 +1005,7 @@ root_info (GnomeVFSMethod *method,
 	   GnomeVFSContext *context)
 {
 	char *path;
+	char *data;
 
 	path =
 	  gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri),
@@ -994,6 +1013,8 @@ root_info (GnomeVFSMethod *method,
 	vfs_file_info->name = (path)?path:g_strdup ("/");
 		
 	g_assert (vfs_file_info->name != NULL);
+	
+	vfs_file_info->valid_fields = 0;
 
 	if (!path || (path[0] == '/' && path [1] == 0)) {
 		vfs_file_info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
@@ -1001,8 +1022,13 @@ root_info (GnomeVFSMethod *method,
 		vfs_file_info->mime_type = g_strdup("x-directory/normal");
 	} else {
 		vfs_file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
-		vfs_file_info->mime_type = g_strdup("application/x-nautilus-link");
-
+		vfs_file_info->mime_type = g_strdup("application/x-gnome-app-info");
+		
+		data = get_workgroup_data (path);
+		vfs_file_info->size = strlen (data);
+		g_free (data);
+		vfs_file_info->valid_fields |=
+		  GNOME_VFS_FILE_INFO_FIELDS_SIZE;
 	}
 
 	vfs_file_info->flags = GNOME_VFS_FILE_FLAGS_NONE;
@@ -1051,6 +1077,7 @@ server_link_info (GnomeVFSMethod *method,
 		  GnomeVFSContext *context)
 {
 	char *share;
+	char *data;
 
 	share =
 	  gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri),
@@ -1059,14 +1086,19 @@ server_link_info (GnomeVFSMethod *method,
 	g_free (share);
 	
 	vfs_file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
-	vfs_file_info->mime_type = g_strdup("application/x-nautilus-link");
+	vfs_file_info->mime_type = g_strdup("application/x-gnome-app-info");
+
+	data = get_computer_data (share);
+	vfs_file_info->size = strlen (data);
+	g_free (data);
 
 	vfs_file_info->flags = 0;
 	
 	vfs_file_info->valid_fields |=
 		GNOME_VFS_FILE_INFO_FIELDS_TYPE |
 		GNOME_VFS_FILE_INFO_FIELDS_FLAGS |
-		GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
+		GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE |
+		GNOME_VFS_FILE_INFO_FIELDS_SIZE;
 	
 	return GNOME_VFS_OK;
 }
@@ -1634,9 +1666,9 @@ browse_read_dir (GnomeVFSMethod *method,
 		
 		if ((content->type == SMB_CONTENT_WORKGROUP) ||
 		    (content->type == SMB_CONTENT_SERVER)) {
-			/* Workgroups and servers are nautilus links */
+			/* Workgroups and servers are desktop files */
 			file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
-			file_info->mime_type = g_strdup("application/x-nautilus-link");
+			file_info->mime_type = g_strdup("application/x-gnome-app-info");
 		} else {
 			/* It's a share */
 			file_info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
